#!/opt/puppetlabs/puppet/bin/ruby
#
#
# See the file "LICENSE" for the full license governing this code.
#
#

require 'json'
require 'open3'
require 'puppet'
Puppet.initialize_settings
require 'easy_type'
require 'puppet_x/enterprisemodules/weblogic/core'


params  = JSON.parse(STDIN.read)
domain  = params.fetch('domain')  { 'default'}
skip    = params.fetch('skip')    { []}
wait    = params.fetch('wait')    { true}.to_s == 'true'
timeout = params.fetch('timeout') { 300 }.to_i

START_STATES = [
  'RUNNING',
  'STARTING',
  'STANDBY',
  'ADMIN',
  'RESUMING',
]

WAIT_IN_BETWEEN = 10  # Wait this time between call's to WLS


begin
  result = {}
  @wls_config = Puppet_X::EnterpriseModules::WebLogic::Tree.new(domain)
  begin
    # Check if the domain exists
    @wls_config.validate
  rescue ArgumentError => e
    result[:_error] = { 
      msg: e.message,
      kind: "enterprisemodules-wls_config/unknown_domain"
    }
    puts result.to_json
    raise e
  end
  admin_server = @wls_config.admin_server
  all_servers  = @wls_config.managed_servers
  servers      = all_servers - [admin_server]
  servers.each do | server|
    state = @wls_config.state_for(server)
    if START_STATES.include?(state)
      result['already_running_managed_servers'] ||= []
      result['already_running_managed_servers'] << server
      next
    end
    if skip.include?(server)
      result['skipped'] ||= []
      result['skipped'] << server
      next
    end
    @wls_config.start_server(server)
    result['started_managed_servers'] ||= []
    result['started_managed_servers'] << server
  end

  if wait
    begin
      Timeout::timeout(timeout) do
        loop do
          states = servers.collect { |server| @wls_config.state_for(server)}
          break if states.all?{|s| s == 'RUNNING'}
          puts 'waiting for all servers to be started...'
          sleep(WAIT_IN_BETWEEN)
        end
      end
    rescue => e
      result[:_error] = { 
        msg: e.message,
        kind: "enterprisemodules-wls_config/timeout"
      }
      puts result.to_json
      raise e
    end
  end

  puts result.to_json

end
