#!/opt/puppetlabs/puppet/bin/ruby
#
#
# See the file "LICENSE" for the full license governing this code.
#
#

require 'json'
require 'open3'
require 'puppet'
Puppet.initialize_settings
require 'easy_type'
require 'puppet_x/enterprisemodules/weblogic/core'

def admin_server_running?(name)
  `pgrep -f "weblogic.Name=#{name}"` != ''
end

params       = JSON.parse(STDIN.read)
domain       = params.fetch('domain')       { 'default'}
admin_server = params.fetch('admin_server') { 'AdminServer'}
wait         = params.fetch('wait')         { true}.to_s == 'true'
timeout      = params.fetch('timeout')      { 300 }.to_i

WAIT_IN_BETWEEN = 10  # Wait this time between call's to WLS

STOP_STATES  = [
  'SHUTTING-DOWN',
  'SUSPENING',
  'FORCE_SUSPEND',
  'SHUTDOWN'
]

begin
  result = {}
  if admin_server_running?(admin_server)
    @wls_config = Puppet_X::EnterpriseModules::WebLogic::Tree.new(domain)
    begin
      # Check if the domain exists
      @wls_config.validate
    rescue ArgumentError => e
      result[:_error] = { 
        msg: e.message,
        kind: "enterprisemodules-wls_config/unknown_domain"
      }
      puts result.to_json
      raise e
    end
    state = @wls_config.state_for(admin_server)
    if STOP_STATES.include?(state)
      result['already_stopped_managed_servers'] ||= []
      result['already_stopped_managed_servers'] << admin_server
    else
      @wls_config.stop_server(admin_server)
      result['stopped_managed_servers'] ||= []
      result['stopped_managed_servers'] << admin_server
    end
    if wait
      begin
        Timeout::timeout(timeout) do
          loop do
            puts 'checking...'
            break if  `pgrep -f "^.*/java -server.* weblogic.Server$"` == ''
            sleep(WAIT_IN_BETWEEN)
          end
        end
      rescue => e
        result[:_error] = { 
          msg: e.message,
          kind: "enterprisemodules-wls_config/timeout"
        }
        puts result.to_json
        raise e
      end
    end
  else
    result['already_stopped_managed_servers'] = [admin_server]
end

puts result.to_json

end
